﻿using System;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Collections.Generic;
using System.Text;
using System.IO;
using SharedDataContract;


namespace ChatApplication
{
    [ServiceContract(CallbackContract = typeof(IChatCallback))]
    public interface IOnlineService
    {
        [OperationContract]
        [FaultContract(typeof(ChatFaultContract))]
        string SaveImageFile(string fileName, Byte[] stream);

        [OperationContract(IsOneWay = false, IsInitiating = true, IsTerminating = false)]
        [FaultContract(typeof(ChatFaultContract))]
        void Join(Person name, Person newperson);


        [OperationContract(IsOneWay = false, IsInitiating = true, IsTerminating = false)]
        [FaultContract(typeof(ChatFaultContract))]
        void AddUser(Person name, Person newperson);

        [OperationContract(IsOneWay = false, IsInitiating = true, IsTerminating = false)]
        [FaultContract(typeof(ChatFaultContract))]
        void RemoveUser(Person name, Person newperson);

        [OperationContract(IsOneWay = false, IsInitiating = true, IsTerminating = false)]
        [FaultContract(typeof(ChatFaultContract))]
        void GetPersonList(Person name);

        [OperationContract(IsOneWay = false)]
        [FaultContract(typeof(ChatFaultContract))]
        void Subscribe(Person activeperson);

        [OperationContract(IsOneWay = false)]
        [FaultContract(typeof(ChatFaultContract))]
        void UnSubscribe(Person activeperson);

        [OperationContract(IsOneWay = false)]
        [FaultContract(typeof(ChatFaultContract))]
        void UnJoin(Person actualperson, Person addedperson);

        [OperationContract(IsOneWay = false)]
        [FaultContract(typeof(ChatFaultContract))]
        void Whisper(Person to, Person from, string msg);

        [OperationContract(IsOneWay = false)]
        [FaultContract(typeof(ChatFaultContract))]
        void PublishVideo(Person personTo, Person personfrom, VideoFrame content);

        [OperationContract]
        [FaultContract(typeof(string))]
        int Increment();

    }
    public interface IChatCallback
    {
        [OperationContract(IsOneWay = true, AsyncPattern = true)]
        IAsyncResult BeginUserEnter(Person person, AsyncCallback callback, object state);
        void EndUserEnter(IAsyncResult result);

        [OperationContract(IsOneWay = true, AsyncPattern = true)]
        IAsyncResult BeginUserActive(Person person, AsyncCallback callback, object state);
        void EndUserActive(IAsyncResult result);



        [OperationContract(IsOneWay = true, AsyncPattern = true)]
        IAsyncResult BeginUserDeActive(Person person, AsyncCallback callback, object state);
        void EndUserDeActive(IAsyncResult result);


        [OperationContract(IsOneWay = true, AsyncPattern = true)]
        IAsyncResult BeginWhisperReceive(Person from, string msg, AsyncCallback callback, object state);
        void EndWhisperReceive(IAsyncResult result);

        [OperationContract(IsOneWay = true, AsyncPattern = true)]
        IAsyncResult BeginReceiveVideo(Person personfrom, VideoFrame content, AsyncCallback callback, object state);
        void EndReceiveVideo(IAsyncResult result);


        [OperationContract(IsOneWay = true, AsyncPattern = true)]
        IAsyncResult BeginAddUserList(Person person, AsyncCallback callback, object state);
        void EndAddUserList(IAsyncResult result);


        [OperationContract(IsOneWay = true, AsyncPattern = true)]
        IAsyncResult BeginUserLeave(Person person, AsyncCallback callback, object state);
        void EndUserLeave(IAsyncResult result);

        //[OperationContract(IsOneWay = true, AsyncPattern = true)]
        //IAsyncResult BeginUserTimOut(Person person, AsyncCallback callback, object state);
        //void EndUserTimeOut(IAsyncResult result);

    }

    public class VideoFrame
    {
        public byte[] Pixels { get; set; }
        public int Height { get; set; }
        public int Width { get; set; }
    }
}
