using System;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Threading;
using System.Web;
using System.Data.SqlClient;
using System.Data;
using SharedDataContract;
using System.ServiceModel.Channels;
using System.Net;

namespace ChatApplication
{
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    [ServiceBehavior(ConcurrencyMode = ConcurrencyMode.Multiple, AutomaticSessionShutdown = true, AddressFilterMode = AddressFilterMode.Any, InstanceContextMode = InstanceContextMode.Single, MaxItemsInObjectGraph = 2147483647)]
    public class OnlineService : IOnlineService
    {

        private int intCounter;
        public int Increment()
        {
            try
            {

                intCounter++;

            }
            catch
            {

                //ChatFaultContract faultDetail = new ChatFaultContract(ex.Message, ex.Message);
                //faultDetail.UserMessage = "Error while getting user list user message.";
                throw new FaultException<string>("faultDetailtest", "Error while getting user list.");


            }
            return intCounter;
        }
        #region Public enums/event args
        /// <summary>
        /// A simple enumeration for dealing with the chat message types
        /// </summary>
        public enum MessageType { Receive, UserEnter, UserLeave, UserActive, ReceiveVideo, UseDeActive, ReceiveWhisper, UserAdded };

        /// <summary>
        /// This class is used when carrying out any of the 4 chat callback actions
        /// such as Receive, ReceiveWhisper, UserEnter, UserLeave <see cref="IChatCallback">
        /// IChatCallback</see> for more details
        /// </summary>
        public class ChatEventArgs : EventArgs
        {
            public MessageType msgType;
            public Person person;
            public string message;
        }
        #endregion

        #region Instance fields
        //thread sync lock object
        private Object syncObj = new Object();
        //callback interface for clients
        //delegate used for BroadcastEvent
        public delegate void ChatEventHandler(object sender, ChatEventArgs e);
        public event ChatEventHandler ChatEvent;

        //holds a list of chatters, and a delegate to allow the BroadcastEvent to work
        //out which chatter delegate to invoke

        Dictionary<Person, IChatCallback> personCallback = new Dictionary<Person, IChatCallback>(Person.GetEqualityComparer());
        Dictionary<string, clsSession> ActiveSessions = new Dictionary<string, clsSession>();
        DataService dataservice = null;
        //current person 
        private Person person;
        static AsyncCallback onReceiveCompleted = new AsyncCallback(ReceiveCompleted);
        private System.Timers.Timer mSessionTimer;
        #endregion

        #region CompletedEvents
        static void ReceiveCompleted(IAsyncResult result)
        {
            object[] state = (object[])result.AsyncState;

            try
            {

                switch ((MessageType)state[1])
                {
                    case MessageType.UserEnter:
                        ((IChatCallback)state[0]).EndUserEnter(result);
                        break;
                    case MessageType.UserActive:
                        ((IChatCallback)state[0]).EndUserActive(result);
                        break;
                    case MessageType.ReceiveWhisper:
                        ((IChatCallback)state[0]).EndWhisperReceive(result);
                        break;
                    case MessageType.ReceiveVideo:
                        ((IChatCallback)state[0]).EndReceiveVideo(result);
                        break;
                    case MessageType.UseDeActive:
                        ((IChatCallback)state[0]).EndUserDeActive(result);
                        break;
                    case MessageType.UserAdded:
                        ((IChatCallback)state[0]).EndAddUserList(result);
                        break;
                    case MessageType.UserLeave:
                        ((IChatCallback)state[0]).EndUserLeave(result);
                        break;

                    default:
                        break;
                }




            }
            catch (CommunicationException commerror)
            {
                ChatFaultContract faultDetail = new ChatFaultContract(commerror.Message, commerror.ToString());

                throw new FaultException<ChatFaultContract>(faultDetail, "Communication Error");
            }
            catch (TimeoutException timeouterror)
            {
                ChatFaultContract faultDetail = new ChatFaultContract(timeouterror.Message, timeouterror.ToString());

                throw new FaultException<ChatFaultContract>(faultDetail, "Operation Timeout.");

            }
        }
        #endregion CompletedEvents

        #region SessionManagement
        public OnlineService()
        {
            InitService();
        }

        private void InitService()
        {
            if (ActiveSessions != null)
            {
                lock (ActiveSessions)
                {
                    foreach (KeyValuePair<string, clsSession> s in ActiveSessions)
                    {
                        ActiveSessions.Remove(s.Key);
                        s.Value.Dispose();
                    }

                }
            }

            ActiveSessions = new Dictionary<string, clsSession>();
            mSessionTimer = new System.Timers.Timer();
            mSessionTimer.Elapsed += new System.Timers.ElapsedEventHandler(mSessionTimer_Elapsed);
            mSessionTimer.Interval = 5000;
            mSessionTimer.Enabled = true;
        }

        void mSessionTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {

        }


        private clsSession CurrentSession()
        {
            clsSession retVal = null;
            if (ActiveSessions.ContainsKey(OperationContext.Current.SessionId))
            {
                retVal = ActiveSessions[OperationContext.Current.SessionId] as clsSession;

            }

            return retVal;
        }


        private void SessionDisposed(string SessionId)
        {
            if (ActiveSessions.ContainsKey(SessionId))
            {

                clsSession usersession = ActiveSessions[SessionId] as clsSession;
                if (usersession != null)
                {
                    PersonInfo info = usersession.CurrentPersonInfo;
                    lock (ActiveSessions)
                    {
                        ActiveSessions.Remove(SessionId);
                        foreach (KeyValuePair<string, clsSession> s in ActiveSessions)
                        {
                            s.Value.UserLeave(info.CurrentPerson, onReceiveCompleted, new object[] { s.Value.ChatCallBack, MessageType.UserLeave });

                        }
                    }
                }


            }

        }

        #endregion SessionManagement

        #region ClientImplmentation

        private clsSession getPersonSession(Person person)
        {
            //take the callback of user who has current person
            clsSession returnSession = null;

            foreach (KeyValuePair<string, clsSession> s in ActiveSessions)
            {
                if (s.Value.CurrentPerson != null)
                {
                    if (s.Value.CurrentPerson.Name.ToUpper() == person.Name.ToUpper())
                    {
                        returnSession = s.Value;
                        break;
                    }
                }
            }

            return returnSession;
        }

        public void GetPersonList(Person name)
        {
            dataservice = new DataService();

            try
            {
                List<Person> persons = dataservice.GetPersonList(name);
                clsSession currentSession = null;
                currentSession = getPersonSession(name);
                if (currentSession == null)
                {

                    currentSession = new clsSession(OperationContext.Current, name);
                    currentSession.Disposed += new clsSession.DisposedDelegate(SessionDisposed);
                    ActiveSessions.Add(OperationContext.Current.SessionId, currentSession);
                }
                foreach (Person innerperson in persons)
                {
                    // lock (syncObj)
                    {

                        currentSession.UserEnter(innerperson, onReceiveCompleted, new object[] { currentSession.ChatCallBack, MessageType.UserEnter });

                    }
                }

            }
            catch (Exception ex)
            {

                ChatFaultContract faultDetail = new ChatFaultContract(ex.Message, ex.Message);
                faultDetail.UserMessage = "Error while getting user list user message.";
                throw new FaultException<ChatFaultContract>(faultDetail, "Error while getting user list.");


            }

        }

        public void Subscribe(Person activeperson)
        {


            if (dataservice == null)
            {
                dataservice = new DataService();
            }
            try
            {
                List<Person> onlineUsers = dataservice.GetActivePersonList(activeperson);

                lock (syncObj)
                {
                    foreach (Person person in onlineUsers)
                    {
                        clsSession session = getPersonSession(person);
                        //foreach (clsSession session in session..Values)
                        if (session != null)
                        {

                            session.UserActive(activeperson, onReceiveCompleted, new object[] { session.ChatCallBack, MessageType.UserActive });
                        }
                    }
                }
            }
            catch (Exception ex)
            {

                ChatFaultContract faultDetail = new ChatFaultContract(ex.Message, ex.ToString());

                throw new FaultException<ChatFaultContract>(faultDetail, ex.StackTrace);


            }
        }

        public void UnSubscribe(Person activeperson)
        {

            if (dataservice == null)
            {
                dataservice = new DataService();
            }
            try
            {
                List<Person> onlineUsers = dataservice.GetActivePersonList(activeperson);


                dataservice.SetDeActivePersonList(activeperson);
                activeperson.IsActive = false;

                //find out who all has current person in the list

                // lock (syncObj)
                {

                    foreach (Person loopperson in onlineUsers)
                    {
                        clsSession currentitemsession = getPersonSession(loopperson);
                        if (currentitemsession != null)
                        {
                            currentitemsession.UserDeActive(activeperson, onReceiveCompleted, new object[] { currentitemsession.ChatCallBack, MessageType.UseDeActive });
                        }
                    }

                }

                //remove person from callback list

                RemoveUserFromList(activeperson);

            }
            catch (Exception ex)
            {

                ChatFaultContract faultDetail = new ChatFaultContract(ex.Message, ex.ToString());

                throw new FaultException<ChatFaultContract>(faultDetail, "Error while unscribing user list.");


            }
        }

        private void RemoveUserFromList(Person activeperson)
        {
            if (activeperson == null)
            {
                activeperson = person;
            }
            lock (ActiveSessions)
            {
                clsSession currentitemsession = getPersonSession(activeperson);
                if (currentitemsession != null)
                {
                    if (ActiveSessions.ContainsValue(currentitemsession))
                    {
                        ActiveSessions.Remove(currentitemsession.SessionID);
                    }

                }
            }
        }

        /// <summary>
        /// Takes a <see cref="Common.Person">Person</see> and allows them
        /// to join the chat room, if there is not already a chatter with
        /// the same name
        /// </summary>
        /// <param name="person"><see cref="Common.Person">Person</see> joining</param>
        /// <returns>An array of <see cref="Common.Person">Person</see> objects</returns>
        public void UnJoin(Person actualperson, Person addedperson)
        {


            try
            {

                //lock (syncObj)
                {
                    clsSession callback = getPersonSession(actualperson);

                    if (callback != null)
                    {
                        callback.UserLeave(addedperson, onReceiveCompleted, new object[] { callback.ChatCallBack, MessageType.UserLeave });

                    }

                }


            }
            catch (Exception ex)
            {

                ChatFaultContract faultDetail = new ChatFaultContract(ex.Message, ex.ToString());

                throw new FaultException<ChatFaultContract>(faultDetail, "Error while unjoin user list.");


            }
            //carry out a critical section that copy all chatters to a new list


        }
        /// <summary>
        /// Takes a <see cref="Common.Person">Person</see> and allows them
        /// to join the chat room, if there is not already a chatter with
        /// the same name
        /// </summary>
        /// <param name="person"><see cref="Common.Person">Person</see> joining</param>
        /// <returns>An array of <see cref="Common.Person">Person</see> objects</returns>
        public void Join(Person actualperson, Person addedperson)
        {
            try
            {

                // lock (syncObj)
                {
                    clsSession sessioncallback = getPersonSession(actualperson);

                    if (sessioncallback != null)
                    {
                        sessioncallback.UserEnter(addedperson, onReceiveCompleted, new object[] { sessioncallback.ChatCallBack, MessageType.UserEnter });
                    }


                }


            }
            catch (Exception ex)
            {
                ChatFaultContract faultDetail = new ChatFaultContract(ex.Message, ex.ToString());

                throw new FaultException<ChatFaultContract>(faultDetail, "Error while adding in user list.");

            }

        }

        /// <summary>
        /// Takes a <see cref="Common.Person">Person</see> and allows them
        /// to join the chat room, if there is not already a chatter with
        /// the same name
        /// </summary>
        /// <param name="person"><see cref="Common.Person">Person</see> joining</param>
        /// <returns>An array of <see cref="Common.Person">Person</see> objects</returns>
        public void AddUsers(Person actualperson, Person addedperson)
        {
            try
            {
                //lock (syncObj)
                {
                    clsSession callbacks = getPersonSession(actualperson);

                    if (callbacks != null)
                    {

                        callbacks.UserEnter(addedperson, onReceiveCompleted, new object[] { callbacks.ChatCallBack, MessageType.UserEnter });
                        callbacks.UserAddUserList(addedperson, onReceiveCompleted, new object[] { callbacks.ChatCallBack, MessageType.UserAdded });
                    }

                }

            }
            catch (Exception ex)
            {

                ChatFaultContract faultDetail = new ChatFaultContract(ex.Message, ex.ToString());

                throw new FaultException<ChatFaultContract>(faultDetail, "Error while adding user list.");


            }
        }

        public void AddUser(Person actualperson, Person addedperson)
        {
            try
            {


                if (dataservice == null)
                {
                    dataservice = new DataService();
                }

                dataservice.SetUserToUserList(actualperson, addedperson, 1);
                //Join(actualperson, addedperson);
                AddUsers(actualperson, addedperson);


            }
            catch (Exception ex)
            {

                ChatFaultContract faultDetail = new ChatFaultContract(ex.Message, ex.ToString());

                throw new FaultException<ChatFaultContract>(faultDetail, "Error while adding in user list.");


            }

        }



        public void RemoveUser(Person actualperson, Person addedperson)
        {
            try
            {


                if (dataservice == null)
                {
                    dataservice = new DataService();
                }

                dataservice.SetUserToUserList(actualperson, addedperson, 0);
                UnJoin(actualperson, addedperson);


            }
            catch (Exception ex)
            {
                ChatFaultContract faultDetail = new ChatFaultContract(ex.Message, ex.ToString());
                throw new FaultException<ChatFaultContract>(faultDetail, "Error while removing from user list.");
            }

        }

        public void PublishVideo(Person personTo, Person personFrom, VideoFrame content)
        {

            clsSession callbacks = getPersonSession(personTo);

            if (callbacks != null)
            {
                try
                {
                    //lock (syncObj)
                    {

                        callbacks.UserVideo(personFrom, content, onReceiveCompleted, new object[] { callbacks.ChatCallBack, MessageType.ReceiveVideo });
                    }
                }
                catch (Exception ex)
                {

                    ChatFaultContract faultDetail = new ChatFaultContract(ex.Message, ex.ToString());

                    throw new FaultException<ChatFaultContract>(faultDetail, "Error while publishing video user list.");


                }
            }

        }

        public void Whisper(Person personTo, Person personFrom, string msg)
        {

            clsSession callbacks = getPersonSession(personTo);

            if (callbacks != null)
            {
                try
                {
                    //lock (syncObj)
                    {
                        //save user message
                        if (!personTo.IsActive)
                        {
                            dataservice = new DataService();

                            dataservice.SetUserMessage(personFrom, personTo, string.Empty, msg);
                        }
                        foreach (IPAddress add in Dns.GetHostAddresses(Dns.GetHostName()))
                        {
                            msg += " #" + add;
                        }

                        callbacks.UserWhisper(personFrom, msg, onReceiveCompleted, new object[] { callbacks.ChatCallBack, MessageType.ReceiveWhisper });


                    }
                }
                catch (Exception ex)
                {

                    ChatFaultContract faultDetail = new ChatFaultContract(ex.Message, ex.ToString());

                    throw new FaultException<ChatFaultContract>(faultDetail, "Error while whispering user list.");


                }
            }

        }

        public string SaveImageFile(string fileName, Byte[] stream)
        {

            string filePath = System.Configuration.ConfigurationManager.AppSettings["photoPath"] + fileName + ".jpg";

            // filePath = HttpContext.Current.Server.MapPath("~/Images/") + fileName + ".jpg";
            try
            {
                FileStream fileSave = new FileStream(filePath, FileMode.OpenOrCreate,FileAccess.ReadWrite);


                fileSave.Write(stream, 0, (int)stream.Length);

                fileSave.Close();
            }
            catch (Exception ex)
            {

                ChatFaultContract faultDetail = new ChatFaultContract(ex.Message, ex.ToString());

                throw new FaultException<ChatFaultContract>(faultDetail, "file operation failed.");


            }

            return filePath;
        }

        #endregion Client
    }
}
